/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: infostats
   Purpose: perform tabular statistical analysis on INFO data file input
  ------------------------------------------------------------------------------
   Usage:   infostats {-d{1-9}} {-foutput_table_name_base} {-r1,2,3-5,...}
                      <info_table_1> <info_item_1> {info_table_2} {info_item_2}
                      {functions--see the aps array for the list} 
  ------------------------------------------------------------------------------
   Calls:   once compiled, nothing--it requires Todd Stellhorn's INFOLIB
   Calling: generally, it should be run from STATS.AML, but it will run from
              the command line
  ------------------------------------------------------------------------------
   Notes:   
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Feb 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
   Alan Brenner      Jun 95      added reselected record support
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include "infolib.h"
#include "infodefs.h"
#include "utils.h"
#include "inforeselect.h"
#include "stats.h"

char debug=0;						    /* set this to a number greater than 0 or run program with -d */

#define STATS 28					    /* total number of statistical operations as listed in aps */
#define SIZE 5						    /* size of the maximum length in the aps array*/

/********************************************************************************************************************************/
int main (argc, argv)					    /*main program function*/
  int argc;
  char *argv[];
{
     char aps[STATS][SIZE]={"ken","spe","pea","svt","dvt","fte","min","max","mea","med","sva","sst","sum","minr","maxr","mod","var","sta","ske","kur","pdt","buk","buu","cuu","lr1","lr2","re1","re2"};
     char **ops;					    /* requested statistical operations; ^ list of available ops */
     long operate;					    /* the number of operation items */

     char *fn1;						    /* INFO table # 1 */
     char *fn2=NULL;					    /* INFO table # 2--default to nothing */
     char *it1;						    /* item from table # 1 */
     char *it2;						    /* second item--may be from table 1 or 2 */
     long cnt1;						    /* number of data items in it1 */
     long cnt2=0;					    /* number of data items in it2 */
     double *ndata1;					    /* numeric data from item 1 */
     double *ndata2;					    /* numeric data from item 2 */
     char **cdata2;					    /* character data from item 2--cases only */
     char **ccases;					    /* list of unique instances in cdata2 */
     double *dcases;					    /* list of unique instances in ndata2 */
     short cdatasize=0;					    /* size of character cases--default to no character cases */
     long cases=0;					    /* number of unique cases (items in ccases and dcases)--default is 0 */
     long *hits;					    /* the number of occurances in each case */
     long *case_index;					    /* case_index[i] is case number for ndata1[i] */

     unsigned long InfoReselect;			    /* number of reselected items */
     InfoReselectStruct *InfoReselects=NULL;		    /* pointer to array of reselected items */

     char basename[11]="STATTEMP";			    /* base name of output INFO tables (set with -f switch) */

     char minswtch=0;					    /* control so that min is dropped if minr is in list */
     char maxswtch=0;					    /* control so that max is dropped if maxr is in list */
     short means=0;					    /* switch for calculation of means--default to not */
     short vars=0;					    /* switch for calculation of variances--default to not */

     long *minrec;					    /* record number of minimum data value */
     long *maxrec;					    /* record number of maximum data value */
     double mean1;					    /* mean of data in item 1--no cases*/
     double mean2;					    /* mean of data in item 2--no cases*/
     double svar1;					    /* sample variance of data in item 1--no cases*/
     double svar2;					    /* sample variance of data in item 2--no cases */
     double none,ntwo,nthr,nfou,nfiv;			    /* temporary space for functions that return several values */
     double *sumd;					    /* sums     of data in ndata1 for each case list in ccases or dcases */
     double *mind;					    /* minimums of data in ndata1 for each case list in ccases or dcases */
     double *maxd;					    /* maximums of data in ndata1 for each case list in ccases or dcases */
     double *meand;					    /* means    of data in ndata1 for each case list in ccases or dcases */
     double *medd;					    /* medians  of data in ndata1 for each case list in ccases or dcases */
     double *moded;					    /* modes    of data in ndata1 for each case list in ccases or dcases */
     double *pvard;					    /* population variances           """""" */
     double *psdd;					    /* population standard deviations """""" */
     double *svard;					    /* sample variances               """""" */
     double *ssdd;					    /* sample standard deviations     """""" */
     double *skewd;					    /* skewness of data in ndata1 for each case list in ccases or dcases */
     double *kurtd;					    /* kurtosis of data in ndata1 for each case list in ccases or dcases */

     long getone();					    /* get one data item */
     long gettwo();					    /* get two data items from the same file */
     void output();					    /* output one line of data, only to INFO */
     void outgoing();					    /* output one line of data to screen and INFO */
     void outrecord();					    /* output one line of data and record number to screen and INFO */
     void outgone();					    /* output one line of bivariate data to screen and INFO */
     void outwent();					    /* output one line of integer integer data to screen and INFO */
     void putout();					    /* output matrix to INFO */

     long count=0,counter;				    /* for loop counters */

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (debug>7)
     {
	  (void) fprintf(stderr,"\nSTART\n");
	  for (counter=0; counter<argc; counter++)
	       (void) fprintf(stderr,"arg %d=%s\n",counter,argv[counter]);
     }

     if (strncmp(argv[0]+strlen(argv[0])-10,"infostatsd",10)==0)
	  debug=9;					    /* set debug output to max if called as infostatd */
     argc--;						    /* adjust the argument list to skip call name */
     argv++;
     
     while ((argc>0) && (argv[0][0]=='-'))		    /* if there are arguements and the first character is a - */
     {
	  switch (argv[0][1])
	  {
	     case 'd':					    /* set the debug value */
	       if (strlen(argv[0])>2)			    /* if there is something after the d */
	       {
		    if (isdigit(argv[0][2])!=0)		    /* if it is a digit */
			 debug=(char) atoi(&argv[0][2]);    /* set the debug value to it */
		    else
			 debug=1;			    /* otherwise minimal debuging */
	       }
	       else					    /* check the next agruement */
	       {
		    if (argc>1)				    /* if there is another argument, */
		    {
			 if (isdigit(argv[1][0])!=0)	    /* is it a number? */
			 {
			      debug=(char) atoi(argv[2]);   /* if yes, set the debug level to it */
			      argc--;			    /* and adjust the argument data to skip it */
			      argv++;
			 }
		    }
		    else
			 debug=1;			    /* not a digit, set minimal debug */
	       }
	       break;

	     case 'f':
	       if (strlen(argv[0])>2)			    /* if there is something after the f */
		    (void) strncpy(basename,&argv[0][2],10); /* copy that as the output file basename */
	       else					    /* check the next agruement */
	       {
		    if (argc>1)				    /* is there another argument? */
		    {					    /* if yes, make it the output file basename */
			 (void) strncpy(basename,argv[1],10);
			 argc--;			    /* and adjust the argument data to skip it */
			 argv++;
		    }
	       }
	       break;

	     case 'r':
	       if (strlen(argv[0])>2)			    /* if there is something after the r */
		    InfoReselect=InfoGetReselects(&(argv[0][2]),&InfoReselects);
	       else					    /* check the next agruement */
	       {
		    if (argc>1)				    /* is there another argument? */
		    {
			 InfoReselect=InfoGetReselects(argv[1],&InfoReselects);
			 argc--;			    /* and adjust the argument data to skip it */
			 argv++;
		    }
	       }
	       break;

	  default:
	       argc=0;					    /* this will force the usage exit */
	  }
	  argc--;					    /* don't count the '-' argument */
	  argv++;					    /* don't look at the '-' argument */
     }

     if (debug>8)
     {
	  for (counter=0; counter<argc; counter++)
	       (void) fprintf(stderr,"arg %d=%s\n",counter,argv[counter]);
     }

     if (argc<1)
     {
	  (void) fprintf(stderr,"Usage: infostats {-d[1-9]} {-foutput_table_basename}\n");
	  error(0,"\t<info_table_1> <item_name_1>\n\t{info_table_2} {item_name_2} {operation(s)}");
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     fn1=malloc(strlen(argv[0])+1);			    /* space for and set required arguments */
     (void) strcpy(fn1,argv[0]);			    /* info_table_1 */
     argc--;						    /* don't count the info_table_1 argument anymore */
     argv++;						    /* don't look at the info_table_1 argument anymore */
     it1=malloc(strlen(argv[0])+1);
     (void) strcpy(it1,argv[0]);			    /* item_name_1 */
     argc--;						    /* don't count the item_name_1 argument anymore */
     argv++;						    /* don't look at the item_name_1 argument anymore */

     if (debug) (void) fprintf(stderr,"filename1=%s \t itemname1=%s\n",fn1,it1);
     if (debug>8)
     {
	  for (counter=0; counter<argc; counter++)
	       (void) fprintf(stderr,"arg %d=%s\n",counter,argv[counter]);
     }

     if (argc>0)					    /* something in info_table_2 field */
     {
	  if (InfoFileExists(argv[0])==INFO_SUCCESS)	    /* if info table specified */
	  {
	       if (debug) (void) fprintf(stderr,"Two variables.\n");
	       fn2=malloc(strlen(argv[0])+1); (void) strcpy(fn2,argv[0]);
	       argc--;					    /* don't count the info_table_2 argument anymore */
	       argv++;					    /* don't look at the info_table_2 argument anymore */
	       it2=malloc(strlen(argv[0])+1); (void) strcpy(it2,argv[0]);
	       argc--;					    /* don't count the info_table_2 argument anymore */
	       argv++;					    /* don't look at the info_table_2 argument anymore */
	       if (debug) (void) fprintf(stderr,"filename2=%s\t itemname2=%s\n",fn2,it2);
	       
	       if (strcmp(fn1,fn2)==0)			    /* two items from same file */
		    cnt2=cnt1=gettwo(fn1,it1,&ndata1,it2,&ndata2,&cdata2,&cdatasize,InfoReselect,&InfoReselects);
	       else
	       {
		    cnt1=getone(fn1,it1,&ndata1,InfoReselect,&InfoReselects); /* get two separate data vectors */
		    cnt2=getone(fn2,it2,&ndata2,InfoReselect,&InfoReselects);
	       }
	       operate=5;				    /* set default operations */
	  }
	  else						    /* no table specified */
	  {
	       char sgnl=0;				    /* control signal, default is no command */
	       char op[SIZE];				    /* temporary lower case of input */

	       for (counter=0; counter<SIZE; counter++)	    /* make sure op is blank */
		    *(op+counter)='\0';
	       for (counter=0; counter<SIZE; counter++)	    /* make sure possible operation is lower case */
		    *(op+counter)=tolower(argv[0][counter]);

	       for (counter=0; counter<STATS; counter++)    /* for every possible operation */
	       {
		    if (strcmp(argv[0],aps[counter])==0)    /* check the current parameter vs operation */
		    {					    /* if they are the same */
			 sgnl=1;			    /* signal that all that follows are operations, not INFO items */
			 break;				    /* exit for loop */
		    }
	       }

	       if (sgnl==0)				    /* argv[0] is NOT an operation */
	       {
		    if (argv[0][0]=='#')		    /* this is an ARC/INFO place holder */
		    {					    /* then just skip this argument */
			 argc--;			    /* don't count the info_table_2 argument anymore */
			 argv++;			    /* don't look at the info_table_2 argument anymore */
		    }

		    if (argv[0][0]!='#')		    /* if item_name_2 argument is not # place-holder */
		    {					    /* then item_name_2 is better be a case item */
			 fn2=malloc(2);
			 (void) strcpy(fn2,"#");	    /* null file name */
			 it2=malloc(strlen(argv[0])+1);
			 (void) strcpy(it2,argv[0]);	    /* copy item name */
			 if (debug) (void) fprintf(stderr,"fn1=%s\tit1=%s\tsingle variable case item\n",fn1,it1);

			 cnt1=gettwo(fn1,it1,&ndata1,it2,&ndata2,&cdata2,&cdatasize,InfoReselect,&InfoReselects);

							    /* get the number of, items per and index of cases */
			 cases=stat_index(cdatasize,cnt1,ndata2,cdata2,&dcases,&ccases,&hits,&case_index);

			 if (debug)
			 {
			      (void) fprintf(stderr,"\nDisplaying data value, its index number and the number of occurances.\n");
			      for (counter=0; counter<cnt1; counter++)
			      {
				   if (cdatasize>0)
					(void) fprintf(stderr,"data value %s = %s",cdata2[counter],ccases[case_index[counter]]);
				   else
					(void) fprintf(stderr,"data value %f = %f",ndata2[counter],dcases[case_index[counter]]);
				   (void) fprintf(stderr,"case:%10d, with %10d occuances.\n",case_index[counter],hits[counter]);
			      }
			 }

			 operate=6;			    /* set default operations */
			 count=8;
		    }
		    argc--;				    /* don't count the item_name_2 argument anymore */
		    argv++;				    /* don't look at the item_name_2 argument anymore */
	       }
	  }
     }
     if (fn2==NULL)					    /* item_name_2 is place holder -- one item only */
     {							    /* or only table name and item name on command line */
	  if (debug) (void) fprintf(stderr,"Single variable.\n");
	  it2=(char *) NULL;
	  cnt1=getone(fn1,it1,&ndata1,InfoReselect,&InfoReselects);  /* get individual item */
	  count=6;					    /* set default operations */
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (cases>0)					    /* for summary items */
     {
	  if (debug) (void) fprintf(stderr,"Getting memory for cased statistics.\n");
	  sumd=calloc((size_t) cases,sizeof(double));	    /* allocate data for floating point output */
	  mind=calloc((size_t) cases,sizeof(double));
	  maxd=calloc((size_t) cases,sizeof(double));
	  meand=calloc((size_t) cases,sizeof(double));
	  medd=calloc((size_t) cases,sizeof(double));
	  moded=calloc((size_t) cases,sizeof(double));
	  pvard=calloc((size_t) cases,sizeof(double));
	  psdd=calloc((size_t) cases,sizeof(double));
	  svard=calloc((size_t) cases,sizeof(double));
	  ssdd=calloc((size_t) cases,sizeof(double));
	  skewd=calloc((size_t) cases,sizeof(double));
	  kurtd=calloc((size_t) cases,sizeof(double));

	  minrec=calloc((size_t) cases,sizeof(long));	    /* allocate space for array of minimum records */
	  maxrec=calloc((size_t) cases,sizeof(long));	    /* allocate space for array of maximum records */
     }
     else
     {
	  if (debug) (void) fprintf(stderr,"Getting memory for uncased statistics.\n");
	  minrec=malloc((size_t) sizeof(long));		    /* allocate space for one minimum record number */
	  maxrec=malloc((size_t) sizeof(long));		    /* allocate space for one maximum record number */
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (debug>7)
     {
	  for (counter=0; counter<argc; counter++)
	       (void) fprintf(stderr,"arg %d=%s\n",counter,argv[counter]);
     }
     if (argc>0)					    /* if operations specified, use them */
     {
	  if (debug) (void) fprintf(stderr,"Creating list of selected operations.\n");
	  operate=argc;
     }
     else						    /* otherwise use these */
     {
	  if (debug) (void) fprintf(stderr,"Creating list of default operations.\n");
	  operate=count;
     }

     if ((ops=calloc((size_t) operate,(size_t) sizeof(char *)))==NULL)
	  error(2,"Unable to allocate memory for operations list.");
     for (counter=0; counter<operate; counter++)
     {
	  if (debug>7) (void) fprintf(stderr,"%d=",counter);
	  if ((ops[counter]=malloc((size_t) SIZE))==NULL)
	       error(2,"Unable to allocate memory for operations list.");
	  if (debug>7) (void) fprintf(stderr,"%p\n",ops[counter]);
     }

     for (counter=0; counter<operate; counter++)	    /* for every operator */
     {
	  long counting;				    /* temporary loop counter */
	  for (counting=0; counting<SIZE; counting++)
	       ops[counter][counting]='\0';		    /* make sure the lines are blank */

	  if (argc>0)					    /* if operations specified, then use them */
	  {
	       for (counting=0; counting<strlen(argv[counter]); counting++)
		    ops[counter][counting]=tolower(argv[counter][counting]);
	  }
	  else						    /* use defaults */
	       (void) strncpy(ops[counter],aps[count+counter],SIZE-1);

	  if (debug>5) (void) fprintf(stderr,"%10d = %s.\n",counter+1,ops[counter]);
     }

     for (count=0; count<operate; count++)		    /* make sure every requested operation is valid */
     {
	  char test=0;					    /* temporary switch, default is not valid */
	  for (counter=0; counter<STATS; counter++)	    /* for every valid operation in list */
	  {
	       if (strcmp(ops[count],aps[counter])==0)	    /* if request is in the list */
		    test=1;				    /* set switch to ok */
	  }

	  if (test!=1)					    /* if test is not ok, then request is not available */
	  {
	       long counting;				    /* temporary for loop counter */

	       (void) fprintf(stderr,"Operation, %s%, not available.\n",ops[count]);
	       for (counting=count; counting<operate-1; counting++)
		    (void) strcpy(ops[counting],ops[counting+1]); /* move list lower */
	       operate--;				    /* decrease total number of available operations by one */
	  }
     }

     for (count=0; count<operate; count++)
     {
	  if (strncmp("min",ops[count],3)==0)		    /* determine if min or minr is called */
	       minswtch++;
	  if (strncmp("max",ops[count],3)==0)		    /* determine if max or maxr is called */
	       maxswtch++;
     }
     if (minswtch>1)					    /* this will happen if both min and minr are called */
     {							    /* min is redundant in this case and must be removed */
	  for (count=0; count<operate; count++)		    /* for every list operation */
	  {
	       if (strncmp("min",ops[count],3)==0 && strncmp("minr",ops[count],4)!=0)
	       {					    /* if this is a min call */
		    for (counter=count; counter<operate-1; counter++)
			 (void) strcpy(ops[counter],ops[counter+1]); /* move operations down one in string array */
		    operate--;				    /* decrement number of operations */
	       }
	  }
     }
     if (maxswtch>1)					    /* this will happen if both max and maxr are called */
     {							    /* max is redundant in this case and must be removed */
	  for (count=0; count<operate; count++)		    /* for every list operation */
	  {
	       if (strncmp("max",ops[count],3)==0 && strncmp("maxr",ops[count],4)!=0)
	       {					    /* if this is a max call */
		    for (counter=count; counter<operate-1; counter++)
			 (void) strcpy(ops[counter],ops[counter+1]); /* move operations down one in string array */
		    operate--;				    /* decrement number of operations */
	       }
	  }
     }

     if (debug)
	  for (counter=0; counter<operate; counter++)
	       (void) fprintf(stderr,"%3d %s\n",counter+1,ops[counter]);

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (cnt2==0 || strcmp(fn1,fn2)==0)			    /* write a bit of possibly useful information */
     {
	  (void) printf("\nThere are %d data values in table %s.\n",cnt1,fn1);
	  output(fn1, "#", "#", "#", "Number of data values", (double) cnt1, INFO_BINARY_TYPE, basename);
	  if (cases>0)
	  {
	       (void) printf("There are %d cases in table %s.\n",cases,fn1);
	       output(fn1, it2, "#", "#", "Number of case items", (double) cases, INFO_BINARY_TYPE, basename);
	  }
     }
     else
     {
	  (void) printf("\nThere are %d data values in table %s.\n",cnt1,fn1);
	  (void) printf("There are %d data values in table %s.\n",cnt2,fn2);
	  output(fn1, "#", "#", "#", "Number of data values", (double) cnt1, INFO_BINARY_TYPE, basename);
	  output(fn2, "#", "#", "#", "Number of data values", (double) cnt2, INFO_BINARY_TYPE, basename);
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     for (count=0; count<operate; count++)		    /* loop every operation requested */
     {
	  if (debug) (void) fprintf(stderr,"\n");

	  if (strncmp("sum",ops[count],3)==0)		    /* find the sum of the data values */
	  {
	       if (cases==0)
	       {
		    outgoing("sum total",fn1,it1,sum(cnt1,ndata1),basename);
		    if (cnt2>0)
			 outgoing("sum total",fn2,it2,sum(cnt2,ndata2),basename);
	       }
	       else
		    sumc(cnt1,ndata1,cases,case_index,&sumd);
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("min",ops[count],3)==0)		    /* find the minimum data value */
	  {
	       if (strncmp("minr",ops[count],4)==0)	    /* recording min record number */
	       {
		    if (cases==0)			    /* no case item */
		    {
			 none=minr(cnt1,ndata1,minrec);
			 outrecord("minimum",fn1,it1,none,*minrec,basename);
			 if (cnt2>0)
			 {
			      none=minr(cnt2,ndata2,minrec);
			      outrecord("minimum",fn2,it2,none,*minrec,basename);
			 }
		    }
		    else				    /* case item */
			 minrc(cnt1,ndata1,cases,case_index,&mind,&minrec);
	       }
	       else
	       {
		    if (cases==0)
		    {
			 outgoing("minimum",fn1,it1,min(cnt1,ndata1),basename);
			 if (cnt2>0)
			      outgoing("mimimum",fn2,it2,min(cnt2,ndata2),basename);
		    }
		    else
			 minc(cnt1,ndata1,cases,case_index,&mind);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("max",ops[count],3)==0)		    /* find the maximum data value */
	  {
	       if (strncmp("maxr",ops[count],4)==0)	    /* recording max record number */
	       {
		    if (cases==0)			    /* no case item */
		    {
			 none=maxr(cnt1,ndata1,minrec);
			 outrecord("maximum",fn1,it1,none,*minrec,basename);
			 if (cnt2>0)
			 {
			      none=maxr(cnt2,ndata2,maxrec);
			      outrecord("maximum",fn2,it2,none,*maxrec,basename);
			 }
		    }
		    else				    /* case item */
		    {
			 maxrc(cnt1,ndata1,cases,case_index,&maxd,&maxrec);
		    }
	       }
	       else
	       {
		    if (cases==0)
		    {
			 outgoing("maximum",fn1,it1,max(cnt1,ndata1),basename);
			 if (cnt2>0)
			      outgoing("maximum",fn2,it2,max(cnt2,ndata2),basename);
		    }
		    else
			 maxc(cnt1,ndata1,cases,case_index,&maxd);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("mea",ops[count],3)==0)		    /* find the mean data value */
	  {
	       if (cases==0)				    /* mean of the entire data set */
	       {
		    if (!means)				    /* if the mean has not already been calculated */
		    {
			 mean1=mean(cnt1,ndata1);	    /* data set 1 */
			 if (cnt2>0)
			      mean2=mean(cnt2,ndata2);	    /* data set 2 */
			 means=1;
		    }
		    outgoing("mean",fn1,it1,mean1,basename); /* print the output */
		    if (cnt2>0)
			 outgoing("mean",fn2,it2,mean2,basename);
	       }
	       else					    /* cased data */
	       {
		    if (!means)				    /* if not already done, calculate it */
		    {
			 meanc(cnt1,ndata1,cases,case_index,hits,&meand);
			 means=1;
		    }
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("med",ops[count],3)==0)		    /* find the median data value */
	  {
	       if (cases==0)
	       {
		    outgoing("median",fn1,it1,median(cnt1,ndata1),basename);
		    if (cnt2>0)
			 outgoing("median",fn2,it2,median(cnt2,ndata2),basename);
	       }
	       else
		    medianc(cnt1,ndata1,cases,case_index,hits,&medd);
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("mod",ops[count],3)==0)		    /* find the modal value */
	  {
	       if (cases==0)
	       {
		    outgoing("mode",fn1,it1,mode(cnt1,ndata1),basename);
		    if (cnt2>0)
			 outgoing("mode",fn1,it2,mode(cnt2,ndata2),basename);
	       }
	       else
		    modec(cnt1,ndata1,cases,case_index,&moded);
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("var",ops[count],3)==0)		    /* find the population variance */
	  {
	       if (cases==0)
	       {
		    if (!means)
		    {
			 mean1=mean(cnt1,ndata1);
			 if (cnt2>0)
			      mean2=mean(cnt2,ndata2);
			 means=1;
		    }
		    outgoing("population variance",fn1,it1,pvar(cnt1,mean1,ndata1),basename);
		    if (cnt2>0)
			 outgoing("population variance",fn2,it2,pvar(cnt2,mean2,ndata2),basename);
	       }
	       else
	       {
		    if (!means)				    /* if not already done, calculate it */
		    {
			 meanc(cnt1,ndata1,cases,case_index,hits,&meand);
			 means=1;
		    }
		    pvarc(cnt1,meand,ndata1,cases,case_index,&pvard);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("sva",ops[count],3)==0)		    /* find the sample variance */
	  {
	       if (cases==0)
	       {
		    if (!means)
		    {
			 mean1=mean(cnt1,ndata1);
			 if (cnt2>0)
			      mean2=mean(cnt2,ndata2);
			 means=1;
		    }
		    if (!vars)
		    {
			 svar1=svar(cnt1,mean1,ndata1);
			 if (cnt2>0)
			      svar2=svar(cnt2,mean2,ndata2);
			 vars=1;
		    }
		    outgoing("sample variance",fn1,it1,svar1,basename);
		    if (cnt2>0)
			 outgoing("sample variance",fn2,it2,svar2,basename);
	       }
	       else
	       {
		    if (!means)				    /* if not already done, calculate it */
		    {
			 meanc(cnt1,ndata1,cases,case_index,hits,&meand);
			 means=1;
		    }
		    if (!vars)				    /* if not already done, calculate it */
		    {
			 svarc(cnt1,meand,ndata1,cases,case_index,&svard);
			 vars=1;
		    }
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("sta",ops[count],3)==0)		    /* find the population standard deviation */
	  {
	       if (cases==0)
	       {
		    if (!means)
		    {
			 mean1=mean(cnt1,ndata1);
			 if (cnt2>0)
			      mean2=mean(cnt2,ndata2);
			 means=1;
		    }
		    outgoing("population standard deviation",fn1,it1,sqrt(pvar(cnt1,mean1,ndata1)),basename);
		    if (cnt2>0)
			 outgoing("population standard deviation",fn2,it2,sqrt(pvar(cnt2,mean2,ndata2)),basename);
	       }
	       else
	       {
		    if (!means)				    /* if not already done, calculate it */
		    {
			 meanc(cnt1,ndata1,cases,case_index,hits,&meand);
			 means=1;
		    }
		    psdc(cnt1,meand,ndata1,cases,case_index,&psdd);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("sst",ops[count],3)==0)		    /* find the sample standard deviation */
	  {
	       if (cases==0)
	       {
		    if (!means)
		    {
			 mean1=mean(cnt1,ndata1);
			 if (cnt2>0)
			      mean2=mean(cnt2,ndata2);
			 means=1;
		    }
		    if (!vars)
		    {
			 svar1=svar(cnt1,mean1,ndata1);
			 if (cnt2>0)
			      svar2=svar(cnt2,mean2,ndata2);
			 vars=1;
		    }
		    outgoing("sample standard deviation",fn1,it1,sqrt(svar1),basename);
		    if (cnt2>0)
			 outgoing("sample standard deviation",fn2,it2,sqrt(svar2),basename);
	       }
	       else
	       {
		    if (!means)				    /* if not already done, calculate it */
		    {
			 meanc(cnt1,ndata1,cases,case_index,hits,&meand);
			 means=1;
		    }
		    ssdc(cnt1,meand,ndata1,cases,case_index,&ssdd);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("ske",ops[count],3)==0)		    /* find the skewness */
	  {
	       if (cases==0)
	       {
		    if (!means)
		    {
			 mean1=mean(cnt1,ndata1);
			 if (cnt2>0)
			      mean2=mean(cnt2,ndata2);
			 means=1;
		    }
		    if (!vars)
		    {
			 svar1=svar(cnt1,mean1,ndata1);
			 if (cnt2>0)
			      svar2=svar(cnt2,mean2,ndata2);
			 vars=1;
		    }
		    outgoing("skewness",fn1,it1,skew(cnt1,mean1,svar1,ndata1),basename);
		    if (cnt2>0)
			 outgoing("skewness",fn2,it2,skew(cnt2,mean2,svar2,ndata2),basename);
	       }
	       else
	       {
		    if (!means)				    /* if not already done, calculate it */
		    {
			 meanc(cnt1,ndata1,cases,case_index,hits,&meand);
			 means=1;
		    }
		    if (!vars)				    /* if not already done, calculate it */
		    {
			 svarc(cnt1,meand,ndata1,cases,case_index,&svard);
			 vars=1;
		    }
		    skewc(cnt1,meand,svard,ndata1,cases,case_index,&skewd);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("kur",ops[count],3)==0)		    /* find the kurtosis */
	  {
	       if (cases==0)
	       {
		    if (!means)
		    {
			 mean1=mean(cnt1,ndata1);
			 if (cnt2>0)
			      mean2=mean(cnt2,ndata2);
			 means=1;
		    }
		    if (!vars)
		    {
			 svar1=svar(cnt1,mean1,ndata1);
			 if (cnt2>0)
			      svar2=svar(cnt2,mean2,ndata2);
			 vars=1;
		    }
		    outgoing("kurtosis",fn1,it1,kurt(cnt1,mean1,svar1,ndata1),basename);
		    if (cnt2>0)
			 outgoing("kurtosis",fn2,it2,kurt(cnt2,mean2,svar2,ndata2),basename);
	       }
	       else
	       {
		    if (!means)				    /* if not already done, calculate it */
		    {
			 meanc(cnt1,ndata1,cases,case_index,hits,&meand);
			 means=1;
		    }
		    if (!vars)				    /* if not already done, calculate it */
		    {
			 svarc(cnt1,meand,ndata1,cases,case_index,&svard);
			 vars=1;
		    }
		    kurtc(cnt1,meand,svard,ndata1,cases,case_index,&kurtd);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("ken",ops[count],3)==0)		    /* find Kendall's tau rank correlation */
	  {
	       if (cnt1==cnt2)
	       {
		    none=kendl(cnt1,ndata1,ndata2,&ntwo,&nthr);
		    outgone("Kendall's tau",fn1,it1,fn2,it2,none,basename);
		    outgone("tau's standard deviations from zero",fn1,it1,fn2,it2,ntwo,basename);
		    outgone("two-sided significance level",fn1,it1,fn2,it2,nthr,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("spe",ops[count],3)==0)		    /* find Spearman's r rank correlation */
	  {
	       if (cnt1==cnt2)
	       {
		    none=spear(cnt1,ndata1,ndata2,&ntwo,&nthr,&nfou,&nfiv);
		    outgone("Spearman's r",fn1,it1,fn2,it2,none,basename);
		    outgone("sum of squared difference of rank",fn1,it1,fn2,it2,ntwo,basename);
		    outgone("standard deviations of squared difference from 0 correlation",fn1,it1,fn2,it2,nthr,basename);
		    outgone("two-sided significance level of squared difference deviation",fn1,it1,fn2,it2,nfou,basename);
		    outgone("two-sided significance level of Spearman's r",fn1,it1,fn2,it2,nfiv,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("pea",ops[count],3)==0)		    /* find Pearson's r linear correlation */
	  {
	       if (cnt1==cnt2)
	       {
		    if (!means)
		    {
			 mean1=mean(cnt1,ndata1);
			 means=1;
		    }
		    mean2=mean(cnt2,ndata2);
		    none=pearson(cnt1,mean1,mean2,ndata1,ndata2,&ntwo,&nthr);
		    outgone("Pearson's r",fn1,it1,fn2,it2,none,basename);
		    outgone("probability of the correlation",fn1,it1,fn2,it2,ntwo,basename);
		    outgone("Fisher's z",fn1,it1,fn2,it2,nthr,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("svt",ops[count],3)==0)		    /* t-test for different mean with same variance */
	  {
	       if (cnt2>0)
	       {
		    none=eqvttest(cnt1,ndata1,cnt2,ndata2,&ntwo);
		    outgone("Student's t for different mean with same variance",fn1,it1,fn2,it2,none,basename);
		    outgone("probability of t",fn1,it1,fn2,it2,ntwo,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("dvt",ops[count],3)==0)		    /* t-test for different mean with different variance */
	  {
	       if (cnt2>0)
	       {
		    none=uevttest(cnt1,ndata1,cnt2,ndata2,&ntwo);
		    outgone("Student's t for different mean with different variance",fn1,it1,fn2,it2,none,basename);
		    outgone("probability of t",fn1,it1,fn2,it2,ntwo,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("pdt",ops[count],3)==0)		    /* t-test for different means of paired data */
	  {
	       if (cnt2>0)
	       {
		    none=pairttest(cnt1,ndata1,ndata2,&ntwo);
		    outgone("Student's t for different mean with paired data",fn1,it1,fn2,it2,none,basename);
		    outgone("probability of t",fn1,it1,fn2,it2,ntwo,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("fte",ops[count],3)==0)		    /* f-test for different variances */
	  {
	       if (cnt2>0)
	       {
		    none=ftest(cnt1,ndata1,cnt1,ndata2,&ntwo);
		    outgone("f test for different variances",fn1,it1,fn2,it2,none,basename);
		    outgone("probability of f",fn1,it1,fn2,it2,ntwo,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("buk",ops[count],3)==0)		    /* chi-square test of measurement vs a theoretical data set */
	  {
	       if (cnt1==cnt2)
	       {
		    none=chisquare1(cnt1,ndata1,ndata2,0,&ntwo,&nthr);
		    outgone("chi-squared test vs theoretical data",fn1,it1,fn2,it2,none,basename);
		    outwent("chi-squared's degrees of freedom",fn1,it1,fn2,it2,(long) ntwo,basename);
		    outgone("probability of chi-square",fn1,it1,fn2,it2,nthr,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("buu",ops[count],3)==0)		    /* chi-square test of measurement vs measurement */
	  {
	       if (cnt1==cnt2)
	       {
		    none=chisquare2(cnt1,ndata1,ndata2,0,&ntwo,&nthr);
		    outgone("chi-squared test of two sample data sets",fn1,it1,fn2,it2,none,basename);
		    outwent("chi-squared's degrees of freedom",fn1,it1,fn2,it2,(long) ntwo,basename);
		    outgone("probability of chi-square",fn1,it1,fn2,it2,nthr,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (strncmp("cuu",ops[count],3)==0)		    /* Kolmogorov-Smirnov test of measurement vs measurement */
	  {
	       if (cnt2>0)
	       {
		    none=ks2(cnt1,ndata1,cnt2,ndata2,&ntwo);
		    outgone("Kolmogorov-Smirnov test of two sample data sets",fn1,it1,fn2,it2,none,basename);
		    outgone("probability of Kolmogorov-Smirnov",fn1,it1,fn2,it2,ntwo,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (stcncmp("lr1",ops[count],3)==0)
	  {
	       if (cnt1==cnt2)
	       {
		    none=linearreg1(cnt1,ndata1,ndata2,&ntwo,&nthr,&nfou,&nfiv);
		    outgone("Linear Regression chi-squaried value",fn1,it1,fn2,it2,none,basename);
		    outgone("Linear Regression slope (m)",fn1,it1,fn2,it2,ntwo,basename);
		    outgone("Linear Regression y intercept (b)",fn1,it1,fn2,it2,nthr,basename);
		    outgone("Linear Regression probable uncertainty of m",fn1,it1,fn2,it2,nfou,basename);
		    outgone("Linear Regression probable uncertainty of b",fn1,it1,fn2,it2,nfiv,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (stcncmp("lr2",ops[count],3)==0)
	  {
	       if (cnt1==cnt2)
	       {
		    none=linearreg1(cnt1,ndata2,ndata1,&ntwo,&nthr,&nfou,&nfiv);
		    outgone("Linear Regression chi-squaried value",fn2,it2,fn1,it1,none,basename);
		    outgone("Linear Regression slope (m)",fn2,it2,fn1,it1,ntwo,basename);
		    outgone("Linear Regression y intercept (b)",fn2,it2,fn1,it1,nthr,basename);
		    outgone("Linear Regression probable uncertainty of m",fn2,it2,fn1,it1,nfou,basename);
		    outgone("Linear Regression probable uncertainty of b",fn2,it2,fn1,it1,nfiv,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (stcncmp("re1",ops[count],3)==0)
	  {
	       if (cnt1==cnt2)
	       {
		    none=robustest(cnt1,ndata1,ndata2,&ntwo,&nthr);
		    outgone("Robust Estimation slope (m)",fn1,it1,fn2,it2,ntwo,basename);
		    outgone("Robust Estimation y intercept (B)",fn1,it1,fn2,it2,nthr,basename);
		    outgone("Mean absolute deviation of the experimental points",fn1,it1,fn2,it2,none,basename);
	       }
	  }

/*------------------------------------------------------------------------------------------------------------------------------*/
	  if (stcncmp("re2",ops[count],3)==0)
	  {
	       if (cnt1==cnt2)
	       {
		    none=robustest(cnt1,ndata2,ndata1,&ntwo,&nthr);
		    outgone("Robust Estimation slope (m)",fn2,it2,fn1,it1,ntwo,basename);
		    outgone("Robust Estimation y intercept (B)",fn2,it2,fn1,it1,nthr,basename);
		    outgone("Mean absolute deviation of the experimental points",fn2,it2,fn1,it1,none,basename);
	       }
	  }
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (cases > 0)
	  putout(fn1,it1,it2,operate,ops,cases,hits,dcases,ccases,cdatasize,sumd,mind,minrec,maxd,maxrec,meand,medd,moded,pvard,psdd,svard,ssdd,skewd,kurtd,basename);

	  return(0);						    /*return no errors*/
}

/********************************************************************************************************************************/
long getone(filename1, itemname1, ndata, reselect, reselects) /* get data from one item */
  char *filename1;					    /* input file name */
  char *itemname1;					    /* input item in filename1 */
  double *ndata[];					    /* array for output data */
  unsigned long reselect;				    /* number of records in reselects */
  InfoReselectStruct **reselects;			    /* record numbers to use */
{
     char skip=0;					    /* record to skip--defualt is no universe polygon record to skip */
     char *cdata=NULL;					    /* pointer for retrieving numeric data from INFO */
     long records;					    /* number of records to be retrieved */
     unsigned long count;				    /* for loop counter */
     double dtest;					    /* value of AREA item in PATs--if >0 then universe polygon record */
     InfoFile *infofile1;				    /* INFO table */
     InfoItemDef *infoitem1;				    /* INFO item in InfoFile */

     if (debug) (void) fprintf(stderr,"\nGetting item, %s, from %s\n",itemname1,filename1);
     if ((infofile1=InfoOpenFile(filename1,InfoREAD))==(InfoFile*)NULL)
	  error(1,"Could not open info file.");
     records=InfoNumberRecords(infofile1);
     if (debug) (void) fprintf(stderr,"total number of records=%d\n",records);

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (reselect==0 && (strstr(filename1,".PAT")!=NULL || strstr(filename1,".pat")!=NULL))
     {							    /* see if .PAT is part of the info file name */
	  if (debug>1) (void) fprintf(stderr,"PAT file name\n",filename1);
	  if (InfoItemExists(infofile1,"AREA")==INFO_SUCCESS || InfoItemExists(infofile1,"area")==INFO_SUCCESS)
	  {						    /* see if AREA is an item of the info file */
	       if (debug>1) (void) fprintf(stderr,"AREA item\n");
	       if ((infoitem1=InfoGetItemDef(infofile1,"AREA"))==(InfoItemDef*)NULL)
		    error(1,"Could not open AREA item.");
	       if (InfoReadRecord(infofile1,1)==INFO_FAIL)
		    error(1,"Error reading info file.");    /* read the first line */
	       if (InfoDecode(infofile1,infoitem1,cdata,&dtest)==INFO_FAIL)
		    error(1,"Error decoding data item.");
	       if (dtest<0.0) skip=1;			    /* universe polygon records in .PATs have areas < 0 */
	       records--;				    /* one less record to be read */
	       if (debug) (void) fprintf(stderr,"AREA=%f\tskip=%d (1==skip universe polygon)\n",dtest,skip);
	  }
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if ((infoitem1=InfoGetItemDef(infofile1,itemname1))==(InfoItemDef*)NULL)
	  error(1,"Could not open data item.");

     if ((*ndata=calloc((size_t) records,sizeof(double)))==NULL)
	  error(1,"Unable to allocate memory for numeric data in function getone.");

/*------------------------------------------------------------------------------------------------------------------------------*/
     for (count=skip; count<records+skip; count++)	    /* read data from info */
     {
	  if (InfoCheckReselect(count+1-skip,*reselects))   /* if use this record */
	  {
	       if (InfoReadRecord(infofile1,count+1)==INFO_FAIL)
		    error(1,"Error reading info file.");    /* read it */
	       if (InfoDecode(infofile1,infoitem1,cdata,*ndata+count)==INFO_FAIL)
		    error(1,"Error decoding data item.");   /* decode it */
	       if (debug>2) (void) fprintf(stderr,"item %d = %lf\n",count+1+skip,*(*ndata+count));
	  }
	  if (InfoMoreReselects(count+1-skip,reselect,reselects)==0)
	       break;					    /* if there are no more records in list, exit for loop */
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (InfoCloseFile(infofile1)==INFO_FAIL)
	  error(1,"Error closing info file.");
     if (reselect==0)
	  return(records);
     else
	  return(reselect);
}

/********************************************************************************************************************************/
long gettwo(filename1, itemname1, data1, itemname2, data2, char2, size2, reselect, reselects) /* get two items from same table */
  char *filename1;					    /* input table name */
  char *itemname1;					    /* input item 1--will always be numeric */
  char *itemname2;					    /* input item 2 */
  char ***char2;					    /* character data from itemname 2 */
  short *size2;						    /* size of string in char2 */
  double *data1[];					    /* data from itemname1 */
  double *data2[];					    /* numeric data from itemname2 */
  unsigned long reselect;				    /* number of records in reselects */
  InfoReselectStruct **reselects;			    /* record numbers to use */
{
     char skip=0;					    /* record to skip--defualt is no universe polygon record to skip */
     char *cdata=NULL;					    /* pointer for retrieving numeric data from INFO */
     long records;					    /* number of records to be retrieved */
     unsigned long count;				    /* for loop counter */
     double dtest;					    /* value of AREA item in PATs--if >0 then universe polygon record */
     InfoFile *infofile1;				    /* INFO table */
     InfoItemDef *infoitem1,*infoitem2;			    /* INFO items in InfoFile */

     if (debug) (void) fprintf(stderr,"\nGetting items, %s  and %s, from %s\n",itemname1,itemname2,filename1);
     if ((infofile1=InfoOpenFile(filename1,InfoREAD))==(InfoFile*)NULL)
	  error(2,"could not open info file.");
     records=InfoNumberRecords(infofile1);
     if (debug) (void) fprintf(stderr,"total number of records=%d\n",records);

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (reselect==0 && (strstr(filename1,".PAT")!=NULL || strstr(filename1,".pat")!=NULL))
     {							    /* see if .PAT is part of the info file name */
	  if (debug>1) (void) fprintf(stderr,"PAT file name\n",filename1);
	  if (InfoItemExists(infofile1,"area")==INFO_SUCCESS || InfoItemExists(infofile1,"AREA")==INFO_SUCCESS)
	  {						    /* see if area is an item of the info file */
	       if (debug>1) (void) fprintf(stderr,"AREA item\n");
	       if ((infoitem1=InfoGetItemDef(infofile1,"area"))==(InfoItemDef*)NULL)
		    error(1,"could not open area item.");
	       if (InfoReadRecord(infofile1,1)==INFO_FAIL)
		    error(1,"error reading info file.");    /* read the first line */
	       if (InfoDecode(infofile1,infoitem1,cdata,&dtest)==INFO_FAIL)
		    error(1,"error decoding data item.");
	       if (dtest<0.0) skip=1;			    /* universe polygon records in  .PATS have areas < 0 */
	       records--;
	       if (debug) (void) fprintf(stderr,"AREA=%f\tskip=%d (1==skip universe polygon)\n",dtest,skip);
	  }
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (debug) (void) fprintf(stderr,"Getting data items.\n");
     if ((infoitem1=InfoGetItemDef(infofile1,itemname1))==(InfoItemDef*)NULL)
	  error(2,"could not open data item.");
     if ((infoitem2=InfoGetItemDef(infofile1,itemname2))==(InfoItemDef*)NULL)
	  error(2,"could not open data item.");

     if (debug) (void) fprintf(stderr,"Getting data memory space.\n");
     if ((*data1=(double *)calloc((size_t) records,sizeof(double)))==NULL)
          error(2,"unable to allocate memory in function gettwo.");
     if ((*data2=(double *)calloc((size_t) records,sizeof(double)))==NULL)
          error(2,"unable to allocate memory in function gettwo.");
     if (InfoItemType(infoitem2)==INFO_CHARACTER_TYPE || InfoItemType(infoitem2)==INFO_DATE_TYPE)
     {
	  if (debug>3) (void) fprintf(stderr,"Getting space for a character case item;");
	  *size2=InfoItemWidth(infoitem2);
	  (*size2)++;
	  if (debug>3) (void) fprintf(stderr,"\t %d bytes per\n",*size2);
	  if ((*char2=calloc((size_t) records,(size_t) sizeof(char *)))==NULL)
	       error(2,"unable to allocate memory in function gettwo.");
	  for (count=0; count<records; count++)
	  {
	       if (debug>5) (void) fprintf(stderr,"%d=",count);
	       if ((*(*char2+count)=malloc((size_t) *size2))==NULL)
		    error(2,"unable to allocate memory in function gettwo.");
	       if (debug>5) (void) fprintf(stderr,"%p\n",*(*char2+count));
	  }
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (debug) (void) fprintf(stderr,"Reading data.\n");
     for (count=skip; count<records+skip; count++)	    /* read data from info */
     {
	  if (InfoCheckReselect(count+1-skip,*reselects))   /* if use this record */
	  {
	       if (InfoReadRecord(infofile1,count+1)==INFO_FAIL)
		    error(2,"error reading info file.");    /* offset by skip to skip universe polygon */
	       if (InfoDecode(infofile1,infoitem1,cdata,*data1+count)==INFO_FAIL)
		    error(2,"error decoding first data item.");
	       if (debug>3) (void) fprintf(stderr,"record %10d = %lf",count+1,*(*data1+count));
	       if (*size2==0)
	       {
		    if (debug>3) (void) fprintf(stderr,"\t%p",*data2+count);
		    if (InfoDecode(infofile1,infoitem2,cdata,*data2+count)==INFO_FAIL)
			 error(2,"error decoding first data item.");
		    if (debug>3) (void) fprintf(stderr,"\t%lf\n",*(*data2+count));
	       }
	       else
	       {
		    if (debug>3) (void) fprintf(stderr,"\t%p",*(*char2+count));
		    if (InfoDecode(infofile1,infoitem2,*(*char2+count),*data2+count)==INFO_FAIL)
			 error(2,"error decoding second data item.");
		    if (debug>3) (void) fprintf(stderr,"\t%s\n",*(*char2+count));
	       }
	  }
	  if (InfoMoreReselects(count+1-skip,reselect,reselects)==0)
	       break;					    /* if there are no more records in list, exit for loop */
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (InfoCloseFile(infofile1)==INFO_FAIL)
	  error(1,"error closing info file.");
     if (debug) (void) fprintf(stderr,"Data read and file closed.\n");
     if (reselect==0)
	  return(records);
     else
	  return(reselect);
}

/********************************************************************************************************************************/
void outgoing(name, tab, itm, data, outname)		    /* print a line to the display and send the data to INFO */
  char *name;						    /* statistic name */
  char *tab;						    /* table containing itm */
  char *itm;						    /* item statistics calculated on */
  double data;						    /* statistic's value */
  char *outname;					    /* basename of output INFO table */
{
     void output();					    /* output one line of data */

     (void) printf("The %s value of %s in %s is %f.\n",name,itm,tab,data);
     output(tab, itm, "#", "#", name, data, INFO_FLOATING_TYPE, outname);
}

/********************************************************************************************************************************/
void outrecord(name, tab, itm, data, record, outname)	    /* print a line of data and the record number */
  char *name;						    /* statistic name */
  char *tab;						    /* table containing itm */
  char *itm;						    /* item statistics calculated on */
  double data;						    /* statistic's value */
  long record;						    /* record number of occurace of data */
  char *outname;					    /* basename of output INFO table */
{
     void output();					    /* output one line of data */

     (void) printf("The %s value of %s in %s is %f.\n",name,itm,tab,data);
     (void) printf("The %s's record number is %d.\n",name,record);
     output(tab, itm, "#", "#", name, data,(int) -record, outname);  /* save the record number with the data */
}

/********************************************************************************************************************************/
void outgone(name,tab1,itm1,tab2,itm2,data, outname)	    /* print bivariate statistic */
  char *name;						    /* statistic name */
  char *tab1;						    /* table containing itm */
  char *itm1;						    /* item statistics calculated on */
  char *tab2;						    /* table containing itm */
  char *itm2;						    /* item statistics calculated on */
  double data;						    /* statistic's value */
  char *outname;					    /* basename of output INFO table */
{
     void output();					    /* output one line of data */

     (void) printf("The %s value of %s in %s\n and %s in %s is %f.\n",name,itm1,tab1,itm2,tab2,data);
     output(tab1,itm1,tab2,itm2,name,data, INFO_FLOATING_TYPE, outname);
}

/********************************************************************************************************************************/
void outwent(name,tab1,itm1,tab2,itm2,data, outname)	    /* print an integer */
  char *name;						    /* statistic name */
  char *tab1;						    /* table containing itm */
  char *itm1;						    /* item statistics calculated on */
  char *tab2;						    /* table containing itm */
  char *itm2;						    /* item statistics calculated on */
  long data;						    /* statistic's value */
  char *outname;					    /* basename of output INFO table */
{
     void output();					    /* output one line of data */

     (void) printf("The %s value of %s in %s\n and %s in %s is %d.\n",name,itm1,tab1,itm2,tab2,data);
     output(tab1,itm1,tab2,itm2,name,(double) data,INFO_BINARY_TYPE,outname);
}

/********************************************************************************************************************************/
void output(filename1, itemname1, filename2, itemname2, operation, data, dtype, outbase)
  char *filename1;					    /* name of first input table */
  char *filename2;					    /* name of second input table, if it exists, otherwise # */
  char *itemname1;					    /* name of item from filename1 */
  char *itemname2;					    /* name of second item (from filename1 or 2) or # if only one item */
  char *operation;					    /* name of the statistic performed */
  int dtype;						    /* output data type (INFO_FLOAT_TYPE or INFO_BINARY_TYPE) */
  double data;						    /* output data (regardless of type) */
  char *outbase;					    /* basename of the output INFO table */
{
     char *cdata=NULL;					    /* NULL data value for writing numeric data to INFO */
     char outname[15];					    /* the name of the output INFO table */
     double outputv=0.0;				    /* NULL data value for writing character data to INFO */
     InfoFile *infofile;				    /* INFO table outname */
     InfoItemDef *tab1,*tab2;				    /* Items holding names of input tables */
     InfoItemDef *itm1,*itm2;				    /* Items holding names of input items */
     InfoItemDef *opr;					    /* Item holding name of the statistic */
     InfoItemDef *fval;					    /* Item holding floating point output */
     InfoItemDef *bval;					    /* Item holding integer output */

/*------------------------------------------------------------------------------------------------------------------------------*/
     (void) strncpy(outname,outbase,10);		    /* construct output file name */
     (void) strcat(outname,".STT");
     if (debug) (void) fprintf(stderr,"\nChecking output table, %s.\n",outname);
     if (InfoFileExists(outname)==INFO_FAIL)
     {
	  if (debug) (void) fprintf(stderr,"Creating output table.\n");
	  if (InfoCreateFile(outname,(InfoFile*)NULL)==INFO_FAIL)
	       error(3,"Could not create output table.");  /* create output table */

	  if (InfoDefineItem(outname,"TABLE_1",16,16,INFO_CHARACTER_TYPE,0)==INFO_FAIL)
	       error(3,"Could not define TABLE_1 in output table.");
	  if (InfoDefineItem(outname,"TABLE_2",16,16,INFO_CHARACTER_TYPE,0)==INFO_FAIL)
	       error(3,"Could not define TABLE_2 in output table.");
	  if (InfoDefineItem(outname,"ITEM_1",16,16,INFO_CHARACTER_TYPE,0)==INFO_FAIL)
	       error(3,"Could not define ITEM_1 in output table.");
	  if (InfoDefineItem(outname,"ITEM_2",16,16,INFO_CHARACTER_TYPE,0)==INFO_FAIL)
	       error(3,"Could not define ITEM_2 in output table.");
	  if (InfoDefineItem(outname,"OPERATION",60,60,INFO_CHARACTER_TYPE,0)==INFO_FAIL)
	       error(3,"Could not define OPERATION in output table.");
	  if (InfoDefineItem(outname,"FVALUE",8,16,INFO_FLOATING_TYPE,6)==INFO_FAIL)
	       error(3,"Could not define floating point FVALUE in output table.");
	  if (InfoDefineItem(outname,"BVALUE",4,10,INFO_BINARY_TYPE,6)==INFO_FAIL)
	       error(3,"Could not define floating point BVALUE in output table.");
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (debug) (void) fprintf(stderr,"Writting items: %s %s %s %s\n",filename1,itemname1,filename2,itemname2);
     if ((infofile=InfoOpenFile(outname,InfoWRITE))==(InfoFile*)NULL)
	  error(3,"Could not open output info file.");	    /* open output table */

     if ((tab1=InfoGetItemDef(infofile,"TABLE_1"))==(InfoItemDef*)NULL)
	  error(3,"Could not open TABLE_1 item.");
     if ((tab2=InfoGetItemDef(infofile,"TABLE_2"))==(InfoItemDef*)NULL)
	  error(3,"Could not open TABLE_2 item.");
     if ((itm1=InfoGetItemDef(infofile,"ITEM_1"))==(InfoItemDef*)NULL)
	  error(3,"Could not open ITEM_1 item.");
     if ((itm2=InfoGetItemDef(infofile,"ITEM_2"))==(InfoItemDef*)NULL)
	  error(3,"Could not open ITEM_2 item.");
     if ((opr=InfoGetItemDef(infofile,"OPERATION"))==(InfoItemDef*)NULL)
	  error(3,"Could not open OPERATION item.");
     if ((fval=InfoGetItemDef(infofile,"FVALUE"))==(InfoItemDef*)NULL)
	  error(3,"Could not open FVALUE item.");
     if ((bval=InfoGetItemDef(infofile,"BVALUE"))==(InfoItemDef*)NULL)
	  error(3,"Could not open BVALUE item.");

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (InfoEncode(infofile,tab1,filename1,outputv)==INFO_FAIL)
	  error(3,"Error encoding TABLE_1 item.");
     if (strcmp(filename2,"#")!=0)
     {
	  if (InfoEncode(infofile,tab2,filename2,outputv)==INFO_FAIL)
	       error(3,"Error encoding TABLE_2 item.");
     }
     if (InfoEncode(infofile,itm1,itemname1,outputv)==INFO_FAIL)
	  error(3,"Error encoding ITEM_1 item.");
     if (strcmp(itemname2,"#")!=0)
     {
	  if (InfoEncode(infofile,itm2,itemname2,outputv)==INFO_FAIL)
	       error(3,"Error encoding ITEM_2 item.");
     }
     if (InfoEncode(infofile,opr,operation,outputv)==INFO_FAIL)
	  error(3,"Error encoding OPERATION item.");

     if (dtype<0)					    /* record record number with data item */
     {
	  if (InfoEncode(infofile,fval,cdata,data)==INFO_FAIL)
	       error(3,"Error encoding FVALUE item.");
	  if (InfoEncode(infofile,bval,cdata,(double) -dtype)==INFO_FAIL)
	       error(3,"Error encoding BVALUE item.");
     }
     else
     {
	  if (dtype==INFO_FLOATING_TYPE || dtype==INFO_NUMBER_TYPE)
	  {						    /* write data to the floating point item */
	       if (InfoEncode(infofile,fval,cdata,data)==INFO_FAIL)
		    error(3,"Error encoding FVALUE item.");
	       if (InfoEncode(infofile,bval,cdata,0.0)==INFO_FAIL)
		    error(3,"Error encoding BVALUE item.");
	  }
	  else
	  {						    /* write data to the binary (integer) item */
	       if (InfoEncode(infofile,fval,cdata,0.0)==INFO_FAIL)
		    error(3,"Error encoding FVALUE item.");
	       if (InfoEncode(infofile,bval,cdata,data)==INFO_FAIL)
		    error(3,"Error encoding BVALUE item.");
	  }
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (InfoWriteRecord(infofile,InfoNumberRecords(infofile)+1)==INFO_FAIL)
	  error(3,"Error writing info file.");
     if (InfoFileFlush(infofile)==INFO_FAIL)		    /* flush and close info file */
	  error(3,"Error flushing info file.");
     if (InfoCloseFile(infofile)==INFO_FAIL)
	  error(3,"Error closing info file.");
}

/********************************************************************************************************************************/
#define CSTATS 12					    /* total number of case statistical operations currently supported */

void putout(fn1, it1, it2, ops, opers, cases, hits, dcases, ccases, cdatasize, su, mi, mir, ma, mar, mn, md, mo, pv, ps, sv, ss, sk, ku, outbase)
  char *fn1;						    /* info table that data was read from */
  char *it1;						    /* data item in fn1 */
  char *it2;						    /* case item in fn1 */
  long ops;						    /* number of operations in opers */
  char **opers;						    /* list of statistical operations performed */
  long cases;						    /* number of unique cases */
  long *hits;						    /* array of number of occurances for each case */
  double *dcases;					    /* array of numeric case names */
  char **ccases;					    /* array of strings of character case names */
  short cdatasize;					    /* length of string in ccases */
  double *su;						    /* array of summations */
  double *mi;						    /* array of minimum values */
  long *mir;						    /* array of record number of occurance of minimum */
  double *ma;						    /* array of maximum values */
  long *mar;						    /* array of record number of occurance of maximum */
  double *mn;						    /* array of means */
  double *md;						    /* array of medians */
  double *mo;						    /* array of modes */
  double *pv;						    /* array of population variances */
  double *ps;						    /* array of population standard deviations */
  double *sv;						    /* array of sample variances */
  double *ss;						    /* array of sample standard deviations */
  double *sk;						    /* array of skewness */
  double *ku;						    /* array of kurtosis */
  char *outbase;					    /* output file name base (before .) */
{
     char output[13];					    /* Name of INFO item in InfoItemDef item[] */
     char outname[15];					    /* the name of the output INFO table */
     short ciw;						    /* case item width */
     short ciow;					    /* case item output width */
     short diow;					    /* data item output width */
     long cit;						    /* case item type */
     short cind;					    /* case item number of decimals */
     short dind;					    /* data item number of decimals */
     short info_out;					    /* output from define item function */
     long count,counter;				    /* for loop counters */
     double hold=0.0;					    /* temporary hold for data going to INFO */
     InfoFile *infofile;				    /* INFO table outname */
     InfoItemDef *caseitem;				    /* case item */
     InfoItemDef *hititems;				    /* number of occurance per case item */
     InfoItemDef *item[CSTATS+2];				    /* output for each possible statistic */

     (void) strncpy(outname,outbase,10);		    /* construct output file name */
     (void) strcat(outname,".STC");

     if (debug)
     {
	  (void) fprintf(stderr,"\nGenerating case output (%s) for:",outname);
	  for (count=0; count<ops; count++) (void) fprintf(stderr," %s",opers[count]);
     }
     
/*------------------------------------------------------------------------------------------------------------------------------*/
     if (debug) (void) fprintf(stderr,"\nGetting case item data from input table, %s.\n",fn1);
     if ((infofile=InfoOpenFile(fn1,InfoREAD))==(InfoFile*)NULL)
	  error(4,"Could not open info data file.");	    /* get case item definition */
     if ((caseitem=InfoGetItemDef(infofile,it2))==(InfoItemDef*)NULL)
	  error(4,"Could not open data file case item.");
     ciw=InfoItemWidth(caseitem);
     ciow=InfoItemOutputWidth(caseitem);
     cit=InfoItemType(caseitem);
     cind=InfoItemNumberDecimals(caseitem);
     for (; cind<0; cind++);				    /* make sure number of decimals is not less than zero */
     if (debug) (void) fprintf(stderr,"Case ItemDef: %s\t%d\t%d\t%d\t%d\n",it2,ciw,ciow,cit,cind);
     if ((caseitem=InfoGetItemDef(infofile,it1))==(InfoItemDef*)NULL)
	  error(4,"Could not open data file data item.");
     diow=InfoItemOutputWidth(caseitem);
     dind=InfoItemNumberDecimals(caseitem);
     for (; dind<0; dind++);				    /* make sure number of decimals is not less than zero */
     diow++; dind++;					    /* increment to show one additional decimal place */
     if (debug) (void) fprintf(stderr,"Data ItemDef: %s\t%d\t%d\n",it1,diow,dind);
     if (InfoCloseFile(infofile)==INFO_FAIL)
	  error(4,"Error closing info file.");

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (InfoFileExists(outname)!=INFO_FAIL)		    /* delete old output file */
	  (void) InfoDeleteFile(outname);

     if (debug) (void) fprintf(stderr,"Creating output table, %s\n",outname);
     if (InfoCreateFile(outname,(InfoFile*)NULL)==INFO_FAIL)
	  error(4,"Could not create output table.");	    /* create output table */

     if (debug) (void) fprintf(stderr,"Defining %s, INSTANCES and output items",it2);
     if (InfoDefineItem(outname,it2,ciw,ciow,cit,cind)==INFO_FAIL)
	  error(4,"Could not define CASE ITEM in output table.");
     if (InfoDefineItem(outname,"INSTANCES",4,10,INFO_BINARY_TYPE,0)==INFO_FAIL)
	  error(4,"Could not define INSTANCES in output table.");

     for (count=0; count<ops; count++)			    /* define an item for each used function */
     {
	  if (debug) (void) fprintf(stderr,".");
	  if (strncmp("sum",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"SUM",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("min",opers[count],3)==0)
	  {
	       info_out=InfoDefineItem(outname,"MINIMUM",8,diow,INFO_FLOATING_TYPE,dind);
	       if (strncmp("minr",opers[count],4)==0)
	       {
		    if ((InfoDefineItem(outname,"MIN_RECORD",4,10,INFO_BINARY_TYPE,0))==INFO_FAIL)
			 error(4,"Could not define floating point item in output table.");
	       }
	  }
	  if (strncmp("max",opers[count],3)==0)
	  {
	       info_out=InfoDefineItem(outname,"MAXIMUM",8,diow,INFO_FLOATING_TYPE,dind);
	       if (strncmp("maxr",opers[count],4)==0)
	       {
		    if ((InfoDefineItem(outname,"MAX_RECORD",4,10,INFO_BINARY_TYPE,0))==INFO_FAIL)
			 error(4,"Could not define floating point item in output table.");
	       }
	  }
	  if (strncmp("mea",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"MEAN",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("med",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"MEDIAN",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("mod",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"MODE",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("var",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"POP_VAR",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("sta",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"POP_SDEV",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("sva",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"SAMP_VAR",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("sst",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"SAMP_SDEV",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("ske",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"SKEWNESS",8,diow,INFO_FLOATING_TYPE,dind);
	  if (strncmp("kur",opers[count],3)==0)
	       info_out=InfoDefineItem(outname,"KURTOSIS",8,diow,INFO_FLOATING_TYPE,dind);
	  if (info_out==INFO_FAIL)
	       error(4,"Could not define floating point item in output table.");
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (debug) (void) fprintf(stderr,"\nOpening output file.\n");
     if ((infofile=InfoOpenFile(outname,InfoWRITE))==(InfoFile*)NULL)
	  error(4,"Could not open output info file.");	    /* open output table */

     if (debug) (void) fprintf(stderr,"Geting %s, INSTANCES and output items:\n",it2);
     if ((caseitem=InfoGetItemDef(infofile,it2))==(InfoItemDef*)NULL)
	  error(4,"Could not open case item.");
     if ((hititems=InfoGetItemDef(infofile,"INSTANCES"))==(InfoItemDef*)NULL)
	  error(4,"Could not open INSTANCES item.");

     for (count=0; count<ops; count++)			    /* get an item for each used function */
     {
	  (void) strcpy(output,"\0\0\0\0\0\0\0\0\0\0\0\0\0");
	  if (strncmp("sum",opers[count],3)==0) (void) strcpy(output,"SUM");
	  if (strncmp("min",opers[count],3)==0)
	  {
	       (void) strcpy(output,"MINIMUM");
	       if (strncmp("minr",opers[count],4)==0)
	       {
		    if (debug) (void) fprintf(stderr,"MIN_RECORD\n");
		    if ((item[CSTATS]=InfoGetItemDef(infofile,"MIN_RECORD"))==INFO_FAIL)
			 error(4,"Could not open MIN_RECORD in output table.");
	       }
	  }
	  if (strncmp("max",opers[count],3)==0)
	  {
	       (void) strcpy(output,"MAXIMUM");
	       if (strncmp("maxr",opers[count],4)==0)
	       {
		    if (debug) (void) fprintf(stderr,"MAX_RECORD\n");
		    if ((item[CSTATS+1]=InfoGetItemDef(infofile,"MAX_RECORD"))==INFO_FAIL)
			 error(4,"Could not open MAX_RECORD in output table.");
	       }
	  }
	  if (strncmp("mea",opers[count],3)==0) (void) strcpy(output,"MEAN");
	  if (strncmp("med",opers[count],3)==0) (void) strcpy(output,"MEDIAN");
	  if (strncmp("mod",opers[count],3)==0) (void) strcpy(output,"MODE");
	  if (strncmp("var",opers[count],3)==0) (void) strcpy(output,"POP_VAR");
	  if (strncmp("sta",opers[count],3)==0) (void) strcpy(output,"POP_SDEV");
	  if (strncmp("sva",opers[count],3)==0) (void) strcpy(output,"SAMP_VAR");
	  if (strncmp("sst",opers[count],3)==0) (void) strcpy(output,"SAMP_SDEV");
	  if (strncmp("ske",opers[count],3)==0) (void) strcpy(output,"SKEWNESS");
	  if (strncmp("kur",opers[count],3)==0) (void) strcpy(output,"KURTOSIS");
	  if (debug) (void) fprintf(stderr,"%s\n",output);
	  if ((item[count]=InfoGetItemDef(infofile,output))==(InfoItemDef*)NULL)
	       error(4,"Could not open output item.");
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     for (counter=0; counter<cases; counter++)
     {
	  if (debug) (void) fprintf(stderr,"Case %10d: ",counter+1);
	  if (cdatasize==0)
	  {						    /* write numeric case id */
	       if (InfoEncode(infofile,caseitem,fn1,dcases[counter])==INFO_FAIL)
		    error(4,"Error encoding CASE_ITEM.");
	       if (debug) (void) fprintf(stderr,"%16f ",dcases[counter]);
	  }
	  else
	  {						    /* write character case id */
	       if (InfoEncode(infofile,caseitem,ccases[counter],hold)==INFO_FAIL)
		    error(4,"Error encoding CASE_ITEM.");
	       if (debug) (void) fprintf(stderr,"%s ",ccases[counter]);
	  }
	  if (InfoEncode(infofile,hititems,fn1, (double) hits[counter])==INFO_FAIL)
	       error(4,"Error encoding INSTANCES.");	    /* write number of occurances */
	  if (debug) (void) fprintf(stderr,"%10d ",hits[counter]);

	  for (count=0; count<ops; count++)		    /* write data for each used function */
	  {
	       if (strncmp("sum",opers[count],3)==0) hold=su[counter];
	       if (strncmp("min",opers[count],3)==0)
	       {
		    hold=mi[counter];
		    if (strncmp("minr",opers[count],4)==0)
		    {
			 if (debug) (void) fprintf(stderr,"%10d ",mir[counter]);
			 if (InfoEncode(infofile,item[CSTATS],fn1, (double) mir[counter])==INFO_FAIL)
			      error(4,"Error encoding MIN_RECORD item.");
		    }
	       }
	       if (strncmp("max",opers[count],3)==0)
	       {
		    hold=ma[counter];
		    if (strncmp("maxr",opers[count],4)==0)
		    {
			 if (debug) (void) fprintf(stderr,"%10d ",mar[counter]);
			 if (InfoEncode(infofile,item[CSTATS+1],fn1, (double) mar[counter])==INFO_FAIL)
			      error(4,"Error encoding MAX_RECORD item.");
		    }
	       }
	       if (strncmp("mea",opers[count],3)==0) hold=mn[counter];
	       if (strncmp("med",opers[count],3)==0) hold=md[counter];
	       if (strncmp("mod",opers[count],3)==0) hold=mo[counter];
	       if (strncmp("var",opers[count],3)==0) hold=pv[counter];
	       if (strncmp("sta",opers[count],3)==0) hold=ps[counter];
	       if (strncmp("sva",opers[count],3)==0) hold=sv[counter];
	       if (strncmp("sst",opers[count],3)==0) hold=ss[counter];
	       if (strncmp("ske",opers[count],3)==0) hold=sk[counter];
	       if (strncmp("kur",opers[count],3)==0) hold=ku[counter];
	       if (debug) (void) fprintf(stderr,"%*.*f ",diow,dind,hold);
	       if (InfoEncode(infofile,item[count],fn1,hold)==INFO_FAIL)
		    error(4,"Error encoding ITEM_1 item.");
	  }
	  if (debug) (void) fprintf(stderr,"\n");
	  if (InfoWriteRecord(infofile,InfoNumberRecords(infofile)+1)==INFO_FAIL)
	       error(4,"Error writing info file.");
     }

/*------------------------------------------------------------------------------------------------------------------------------*/
     if (InfoFileFlush(infofile)==INFO_FAIL)		    /* flush and close info file */
	  error(4,"Error flushing info file.");
     if (InfoCloseFile(infofile)==INFO_FAIL)
	  error(4,"Error closing info file.");
}
