/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: indec()
   Purpose: perform tabular statistical analysis on INFO data file input
  ------------------------------------------------------------------------------
   Usage:   
  ------------------------------------------------------------------------------
   Calls:   once compiled, nothing
   Calling: any program that needs the case item statical operations
  ------------------------------------------------------------------------------
   Notes:   
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 95      original coding
     alan_brenner@unixmail.edi.epa.gov
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/
#include "stats.h"

long stat_index(cdatasize, rows, ndata, cdata, ncases, ccases, hits, case_index)
  short cdatasize;					    /* >0 for character data, otherwise 0 */
  long rows;						    /* number of data items */
  double *ndata;					    /* array of numeric data */
  char **cdata;						    /* array of strings of character data */
  double **ncases;					    /* pointer to array of numeric cases--output */
  char ***ccases;					    /* pointer to array of string cases--output */
  long **case_index;						    /* pointer to array of case membership--output */
  long **hits;						    /* pointer to array of events per case--output */
{
     long cases=1;					    /* number of unique cases in ndata or cdata--output (at least 1) */
     long counter,counting;				    /* for loop counters */
     long *events;					    /* temporary hold for calculating **hits */
     
     if (debug) (void) fprintf(stderr,"\nFinding cases and generating index.\n");
     if (debug) (void) fprintf(stderr,"Getting memory for temporary number of items per case (events).\n");
     if ((events=calloc((size_t) rows,(size_t) sizeof(long)))==NULL)
	  error(2,"Unable to allocate memory for temporary number of items per case.");
     for (counter=0; counter<rows; counter++)		    /* allocated to maximum possible size--every item is unique */
	  events[counter]=1;				    /* every event will have at least one */

/*********************************************************************************************************************************/
     if (cdatasize>0)					    /* character case data */
     {
	  char *temp_data;				    /* temporary copy of case data */
	  char **temp_cases;				    /* temporary copy of output cases */
	  int (*cchoose)()=strcmp;			    /* selection function for qsort */

	  if (debug>7)
	  {
	       (void) fprintf(stderr,"Input data:\n");
	       for (counter=0; counter<rows; counter++)
		    (void) fprintf(stderr,"item %10d = %1s.\n",counter+1,cdata[counter]);
	  }

	  if (debug) (void) fprintf(stderr,"Getting memory for temporary copy of character data:");
	  if ((temp_data=malloc((size_t) (rows*cdatasize)))==NULL)
	       error(2,"Unable to allocate memory for temporary copy of character data.");
	  if (debug) (void) fprintf(stderr,"\t #ofbytes=%d\t\t%p\n",cdatasize,temp_data);

	  for (counter=0; counter<rows*cdatasize; counter++)
	       *(temp_data+counter)='\0';		    /* make sure the memory is NUL */

	  if (debug) (void) fprintf(stderr,"Making a temporary copy of the data.\n");
	  for (counter=0; counter<rows; counter++)
	       (void) strncpy(temp_data+counter*cdatasize,cdata[counter],cdatasize-1);
	  if (debug>6)					    /* print the data that will be sorted */
	  {
	       for (counter=0; counter<rows; counter++)
		    (void) fprintf(stderr,"%s\t%s\n",cdata[counter],temp_data+counter*cdatasize);
	  }

	  if (debug) (void) fprintf(stderr,"Sorting data.\n");
	  qsort(temp_data,(size_t) rows,(size_t) cdatasize,cchoose);
	  if (debug>5)					    /* print sorted data */
	  {
	       for (counter=0; counter<rows; counter++)
		    (void) fprintf(stderr,"%s\t%s\n",cdata[counter],temp_data+counter*cdatasize);
	  }

	  if (debug) (void) fprintf(stderr,"Getting memory for temporary character cases.\n");
							    /* get memory for pointers to strings (pointers to pointers) */
	  if ((temp_cases=calloc((size_t) rows,(size_t) sizeof(char *)))==NULL)
	       error(2,"Unable to allocate memory for temporary character cases.");

	  for (counter=0; counter<rows; counter++)	    /* get memory for strings (pointers to type character) */
	  {
	       if (debug>8) (void) fprintf(stderr,"%10d = ",counter+1);
	       if ((temp_cases[counter]=malloc((size_t) cdatasize))==NULL)
		    error(2,"Unable to allocate memory for temporary character cases.");

	       for (counting=0; counting<cdatasize; counting++)
		    *(temp_cases[counter]+counting)='\0';   /* make sure it is blank */
	       if (debug>8) (void) fprintf(stderr,"%p\n",temp_cases[counter]);
	  }

	  if (debug) (void) fprintf(stderr,"Generating list of unique character cases.\n");
	  (void) strncpy(temp_cases[0],temp_data,cdatasize-1); /* first word is unique */
	  if (debug>6) (void) fprintf(stderr,"Case %d = %s.\n",1,temp_data);

	  for (counter=1; counter<rows; counter++)	    /* for every other string */
	  {
	       if (strncmp(temp_data+counter*cdatasize,temp_cases[cases-1],cdatasize-1)!=0)
	       {					    /* the current list item is not the same as the summary list item */
		    (void) strncpy(temp_cases[cases],temp_data+counter*cdatasize,cdatasize-1);
		    cases++;				    /* increment total number of summary items and copy new item to list */
		    if (debug>6) (void) fprintf(stderr,"Case %d = %s\n",cases,temp_cases[cases-1]);
	       }
	       else
		    events[cases-1]++;			    /* not new, so increment count of current item */
	  }

	  if (debug) (void) fprintf(stderr,"Getting memory for output character cases.\n");
	  if ((*ccases=calloc((size_t) cases,(size_t) sizeof(char *)))==NULL)
	       error(2,"Unable to allocate memory for character cases.");

	  for (counter=0; counter<cases; counter++)	    /* get memory for strings */
	  {
	       if (debug>5) (void) fprintf(stderr,"%10d =",counter+1);
	       if ((*((*ccases)+counter)=malloc((size_t) cdatasize))==NULL)
		    error(2,"Unable to allocate memory for character cases.");

	       for (counting=0; counting<cdatasize; counting++)
		    *(*((*ccases)+counter)+counting)='\0';   /* make sure it is blank, then copy the case into it */
	       (void) strncpy(*(*ccases+counter),temp_cases[counter],cdatasize-1);

	       if (debug>5) (void) fprintf(stderr," %s\n",*(*ccases+counter));
	  }
     }
/*********************************************************************************************************************************/
     else						    /* numeric case data */
     {
	  int (*dchoose)()=dpick;			    /* selection function for qsort */
	  double *temp_data;				    /* temporary copy of data */
	  double *temp_cases;				    /* temporary copy of output cases */

	  if (debug>7)					    /* print input data */
	  {
	       (void) fprintf(stderr,"Input data:\n");
	       for (counter=0; counter<rows; counter++)
		    (void) fprintf(stderr,"item %10d = %16.6f\n",counter+1,ndata[counter]);
	  }

	  if (debug) (void) fprintf(stderr,"Getting memory for temporary copy of numeric data.\n");
	  if ((temp_data=calloc((size_t) rows,(size_t) sizeof(double)))==NULL)
	       error(2,"Unable to allocate memory for temporary copy of numeric data.");

	  if (debug) (void) fprintf(stderr,"Making a temporary copy of the data.\n");
	  for (counter=0; counter<rows; counter++)
	       temp_data[counter]=ndata[counter];	    /* make a local copy for sorting */
	  if (debug>6)					    /* print the data that will be sorted */
	  {
	       for (counter=0; counter<rows; counter++)
		    (void) fprintf(stderr,"%f\t%f\n",ndata[counter],temp_data[counter]);
	  }

	  if (debug) (void) fprintf(stderr,"Sorting data.\n");
	  qsort(temp_data,rows,sizeof(double),dchoose);	    /* sort it */
	  if (debug>5)					    /* print the sorted data */
	  {
	       for (counter=0; counter<rows; counter++)
		    (void) fprintf(stderr,"%f\t%f\n",ndata[counter],temp_data[counter]);
	  }

	  if (debug) (void) fprintf(stderr,"Getting memory for temporary copy of numeric cases.\n");
	  if ((temp_cases=calloc((size_t) rows,(size_t) sizeof(double)))==NULL)
	       error(2,"Unable to allocate memory for temporary copy of character data.");

	  if (debug) (void) fprintf(stderr,"Generating list of unique numeric cases.\n");
	  temp_cases[0]=temp_data[0];			    /* first value is always unique */
	  if (debug>6) (void) fprintf(stderr,"Case %10d = %f\n",1,temp_cases[0]);
	  for (counter=1; counter<rows; counter++)
	  {						    /* for every other data item */
	       if (temp_data[counter]!=temp_cases[cases-1])
	       {					    /* is it new? */
		    temp_cases[cases]=temp_data[counter];   /* if yes, increase then number of unique items & save the value */
		    cases++;
		    if (debug>6) (void) fprintf(stderr,"Case %10d = %f\n",cases,temp_cases[cases-1]);
	       }
	       else
		    events[cases-1]++;	    /* not new, so increase the number of instances of the current item */
	  }

	  if (debug) (void) fprintf(stderr,"Getting memory for output copy of numeric cases.\n");
	  if ((*ncases=calloc((size_t) cases,(size_t) sizeof(double)))==NULL)
	       error(2,"Unable to allocate memory for temporary copy of character data.");
	  for (counter=0; counter<cases; counter++)	    /* allocation can be no larger than row, likely smaller */
	  {
	       *(*ncases+counter)=temp_cases[counter];    /* assign unique case to output array */
	       if (debug>5) (void) fprintf(stderr,"%10d = %f\n",counter+1,*(*ncases+counter));
	  }
     }

/*********************************************************************************************************************************/
     if (debug) (void) fprintf(stderr,"Getting memory for output number of items per case (hits).\n");
     if ((*hits=calloc((size_t) cases,(size_t) sizeof(long)))==NULL)
	  error(2,"Unable to allocate memory for output copy of occurance data.");
     for (counter=0; counter<cases; counter++)		    /* allocation can be no larger than row, and may be smaller */
     {
	  *(*hits+counter)=events[counter];		    /* copy actual count to output */

	  if (debug>3)					    /* print debuging data */
	  {
	       if (cdatasize>0)
		    (void) fprintf(stderr,"Case %10d = %s",counter+1,*(*ccases+counter));
	       else
		    (void) fprintf(stderr,"Case %10d = %f",counter+1,*ncases+counter);
	       (void) fprintf(stderr,"\twith %10d occurance(s)\n",*(*hits+counter));
	  }
     }
	  
/*********************************************************************************************************************************/
     if (debug) (void) fprintf(stderr,"Getting memory for output case index.\n");
     if ((*case_index=calloc((size_t) rows,(size_t) sizeof(long)))==NULL)
	  error(2,"Unable to allocate memory for output copy of case index.");

     for (counter=0; counter<rows; counter++)		    /* for each data item */
     {
	  for (counting=0; counting<cases; counting++)	    /* find the case for each record */
	  {
	       if (cdatasize>0)				    /* character cases */
	       {
		    if (strncmp(cdata[counter],*(*ccases+counting),cdatasize-1)==0)
		    {					    /* this value is part of this case? */
			 *(*case_index+counter)=counting;   /* then let index point at this case */
			 if (debug>2)
			      (void) fprintf(stderr,"%s = %s %10d\n",cdata[counter],*(*ccases+counting),*(*case_index+counter));
			 break;
		    }
	       }
	       else					    /* numeric cases */
	       {
		    if (ndata[counter]==*(*ncases+counting))
		    {					    /* this value is part of this case? */
			 *(*case_index+counter)=counting;   /* then let index point at this case */
			 if (debug>2)
			      (void) fprintf(stderr,"%f = %f %10d\n",ndata[counter],*(*ncases+counting),*(*case_index+counter));
			 break;
		    }
	       }
	  }
     }

     return(cases);
}
